#!/opt/nimsoft/bin/perl
use strict;
use warnings;

use English qw( -no_match_vars );
use Carp;
use Getopt::Long;

use Data::Dumper;
use lib qw( /opt/nimsoft/perllib );

use Nimbus::API;
use Nimbus::PDS;

my $address  = "controller"; # probe address: /domain/hub/robot/probe
my $username = "administrator";

GetOptions(
	   "address=s"  => \$address,
	   "username=s" => \$username,
)
  or croak( "$PROGRAM_NAME:  Invalid options" );

local $OUTPUT_AUTOFLUSH = 1;
print "password: ";
my $password = <>;
chomp( $password );

my $login = nimLogin( $username, $password );

# probe-commands takes named parameters via a PDS
# probe_list takes two: name and robot
my $pdsIn = pdsCreate();
pdsPut_PCH( $pdsIn, name       => '' );
pdsPut_PCH( $pdsIn, robot      => '' );

# nimRequest(), nimNamedRequest() or nimSessionRequest()
my ( $rc, $rdata ) = nimNamedRequest( $address, probe_list => $pdsIn );
croak( "$PROGRAM_NAME: nimNamedRequest( $address, probe_list => ... ) failed: rc=$rc" )
    if $rc != 0;

print "#\n# PDS Dump:\n#\n";
pdsDump( $rdata );

# Create a "Nimbus::PDS" object, initiazed with the $rdata.
# That way we can access methods described/available in Nimbus::PDS
my $pds = Nimbus::PDS->new( $rdata );


# one of these methods converts the PDS to a HashRef
my $probe_list = $pds->asHash();

pdsDelete( $rdata );		# we don't need the PDS anymore

print "#\n# HashRef Dump:\n#\n";
print Data::Dumper->		# print a dump
  new( [ $probe_list ] )->	# of the HashRef $probe_list
  Sortkeys( 1 )->               # sort the hash-keys
  Indent( 1 )->                 # indent consitently
  Dump;


print "#\n# Access a key\n#\n";
print "probe_list->{ cdm }{ description } = '",  $probe_list->{ cdm }{ description },   "'\n";

=pod

=encoding utf8

=head1 NAME

probe_list - probe_list from Nimbus perl

=head1 SYNOPSIS

probe_list [ --username USERNAME ] [ --address /DOMAIN/HUB/ROBOT ]

=head1 DESCRIPTION

=head2 ARGUMENTS

=over 4

=item B<--username> I<USERNAME>

Default username is C<administrator>

=item B<--address> I</DOMAIN/HUB/ROBOT>

Nimbus address of the controller to contact, default is the one  assumed to be  running  on  localhost.

=back

The data-structure in a PDS and in a perl hash and also in YAML and JSON as all based on the same ideas.

The most important is the    key => value  representation of data.

=cut

=head1 EXAMPLE OUTPUT

Part of a dump from my site:

  #
  # PDS Dump:
  #
  controller      PDS_PDS         395 
   name            PDS_PCH          11 controller
   description     PDS_PCH          34 Robot process and port control~
   group           PDS_PCH          15 Infrastructure
   active          PDS_I             2 1
   type            PDS_I             2 0
   command         PDS_PCH          11 controller
   config          PDS_PCH          10 robot.cfg
   .
   .
   .
   pkg_name        PDS_PCH          10 processes
   pkg_version     PDS_PCH           5 4.01
   pkg_build       PDS_PCH           3 02
  #
  # HashRef Dump:
  #
  $VAR1 = {
    .
    .
    .
      'workdir' => 'probes/system/cdm'
    },
    'controller' => {
      'active' => '1',
      'command' => 'controller',
      'config' => 'robot.cfg',
      'description' => 'Robot process and port controller',
      'group' => 'Infrastructure',
      'last_action' => '0',
      'last_started' => '1454378321',
      'logfile' => '/opt/nimsoft/log/controller.log',
      'name' => 'controller',
      'pid' => '1099',
      'pkg_name' => 'robot_update',
      'pkg_version' => '7.63',
      'times_activated' => '0',
      'times_started' => '1',
      'timespec' => '',
      'type' => '0',
      'workdir' => 'robot'
    },
    'hdb' => {
      'active' => '1',
    .
    .
    .
  };
  #
  # Access a key
  #
  probe_list->{ cdm }{ description } = 'CPU, Disk and Memory performance probe'

=head1 SEE ALSO

  /opt/nimsoft/bin/pu
  perldoc /opt/nimsoft/perllib/Nimbus/API.pm
  perldoc /opt/nimsoft/perllib/Nimbus/PDS.pm
  Nimsoft Controller Technical Brief, Version: 1.01

=cut

__END__
